///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// Copyright Nurve Networks LLC 2009
// 
// Filename: CHAM_PIC_FLASH_DRV_V010.C
//
// Original Author: Andre' LaMothe
//                  Joshua Hintze
// 
// Last Modified: 8.28.09
//
// Description:  External FLASH driver. Not written yet.
//  
// Overview:
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES ///////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// include local Chameleon PIC API header files
#include "CHAM_PIC_SYSTEM_V010.h"
#include "CHAM_PIC_FLASH_DRV_V010.h"
#include "CHAM_PIC_I2C_SPI_DRV_V010.h"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MACROS /////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES/////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// TYPES/CLASSES //////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS ////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// conditional compilation for FLASH memory variants
#if (FLASH_ROM == AT26F004)

// we could use PGM memory for these and or do it algorithmically, but this is cleaner for now
// 11 sectors [64K, 64K, 64K, 64K, 64K, 64K, 64K, 32K, 8K, 8k, 16K ]
//            |<------- (7) 64K sectors ------->|  1      2     1 



    long sector_addresses[NUM_FLASH_ROM_SECTORS] = {    0x00000,   // 64K sector 0
                                                        0x10000,   // 64K sector 1
                                                        0x20000,   // 64K sector 2
                                                        0x30000,   // 64K sector 3
                                                        0x40000,   // 64K sector 4
                                                        0x50000,   // 64K sector 5
                                                        0x60000,   // 64K sector 6
                                                        0x70000,   // 32K sector 7
                                                        0x78000,   // 8K sector 8  
                                                        0x7A000,   // 8K sector 9
                                                        0x7D000 }; // 8K sector 10
#endif


#if (FLASH_ROM == AT26DF081A)

// we could use PGM memory for these and or do it algorithmically, but this is cleaner for now
// 19 sectors [64K, 64K, 64K, ..., 64K, 64K, 64K, 64K, 16K, 8K, 8k, 32K ]
//            |<----------- (15) 64K sectors -------->| 1     2     1 


    long sector_addresses[NUM_FLASH_ROM_SECTORS] =    { 0x00000,   // 64K sector 0 
                                                        0x10000,   // 64K sector 1
                                                        0x20000,   // 64K sector 2 
                                                        0x30000,   // 64K sector 3
                                                        0x40000,   // 64K sector 4
                                                        0x50000,   // 64K sector 5 
                                                        0x60000,   // 64K sector 6
                                                        0x70000,   // 64K sector 7 
                                                        0x80000,   // 64K sector 8
                                                        0x90000,   // 64K sector 9
                                                        0xA0000,   // 64K sector 10
                                                        0xB0000,   // 64K sector 11
                                                        0xC0000,   // 64K sector 12
                                                        0xD0000,   // 64K sector 13
                                                        0xE0000,   // 64K sector 14

                                                        0xF0000,   // 16K sector 15
                                                        0xF4000,   //  8K sector 16
                                                        0xF6000,   //  8K sector 17

                                                        0xF8000 }; // 32K sector 18
#endif


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// EXTERNALS //////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// PROTOTYPES /////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCTIONS  /////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// flash memory commands (not implmented yet)
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Flash_Open()
{
// In the flash open function we will be unprotecting all of the sectors on the flash chip.

int index;


for (index = 0; index < NUM_FLASH_ROM_SECTORS; index++)
    {
	// step 1: send WRITE_ENABLE command
	
	// set CS to SPI select channel 3 (FLASH)
	SPI_SET_CS(SPI_CS_FLASH);
	SPI_WriteRead( WRITE_ENABLE );
	
	// disable SPI interface
	// set CS to SPI select channel 0 (null)
	SPI_SET_CS(SPI_CS_0);

	DELAY_MS(10);

	// step 2: unprotect all sectors from sector address list

    // enable SPI interface
    // set CS to SPI select channel 3 (FLASH)
    SPI_SET_CS(SPI_CS_FLASH);

    SPI_WriteRead( UNPROTECT_SECTOR );

    // write sector address, there are only a handful of sectors since each is 4-64K
    SPI_WriteRead( sector_addresses[ index ] >> 16 );
    SPI_WriteRead( sector_addresses[ index ] >> 8 );
    SPI_WriteRead( sector_addresses[ index ] >> 0 );

    // disable SPI interface
    // set CS to SPI select channel 0 (null)
    SPI_SET_CS(SPI_CS_0);

    DELAY_MS(10);

    } // end for index


// return success
return(1);

} // end Flash_Open

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Flash_Close()
{
// In the flash close function we will be protecting all of the sectors on the flash chip.

int index;

for (index = 0; index < NUM_FLASH_ROM_SECTORS; index++)
    {
	// step 1: send WRITE_ENABLE command
	
	// set CS to SPI select channel 3 (FLASH)
	SPI_SET_CS(SPI_CS_FLASH);
	SPI_WriteRead( WRITE_ENABLE );
	
	// disable SPI interface
	// set CS to SPI select channel 0 (null)
	SPI_SET_CS(SPI_CS_0);

	DELAY_MS(10);

	// step 2: protect all sectors from sector address list

    // enable SPI interface
    // set CS to SPI select channel 3 (FLASH)
    SPI_SET_CS(SPI_CS_FLASH);

    SPI_WriteRead( PROTECT_SECTOR );

    // write sector address, there are only a handful of sectors since each is 4-64K
    SPI_WriteRead( sector_addresses[ index ] >> 16 );
    SPI_WriteRead( sector_addresses[ index ] >> 8 );
    SPI_WriteRead( sector_addresses[ index ] >> 0 );

    // disable SPI interface
    // set CS to SPI select channel 0 (null)
    SPI_SET_CS(SPI_CS_0);

    DELAY_MS(10);

    } // end for index


// return success for now
return(1);

} // end Flash_Close

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Flash_Write(unsigned long Address, unsigned char *Bytes, unsigned int NumBytes)
{
// The Address argument is the flat memory address 0x00000->0xFFFFF in which to write. The
// Bytes array must be provided already by the calling function and will be used when writting
// to the flash memory. 
//
// This function assumes that the area has previously been erased and that the sector has already been 
// unlocked (i.e. Flash_Open) has been called. Also if you write beyond program memory upper address
// programing or unexpected results can happen.

// Simple test for address out of bounds
if(Address > 0xFFFFF) return 0;

unsigned long flash_start_addr;
unsigned int flash_count, index;

// set CS to SPI select channel 3 (FLASH)
SPI_SET_CS(SPI_CS_FLASH);
SPI_WriteRead( WRITE_ENABLE );

// disable SPI interface
// set CS to SPI select channel 0 (null)
SPI_SET_CS(SPI_CS_0);

DELAY_MS(1);

// set CS to SPI select channel 3 (FLASH)
SPI_SET_CS(SPI_CS_FLASH);

flash_start_addr = Address;
flash_count      = NumBytes; 


// initiate seq program command
SPI_WriteRead( SEQ_PROGRAM );

// write starting address byte by byte, MSB first...
SPI_WriteRead( flash_start_addr >> 16 );
SPI_WriteRead( flash_start_addr >> 8 );
SPI_WriteRead( flash_start_addr >> 0 );

// write first byte
SPI_WriteRead( *Bytes++ );

// de-assert set CS to SPI select channel 0 (null)
SPI_SET_CS(SPI_CS_0);

// Delay to make sure the single byte was written
DELAY_US(10);

// now write remaining data
for (index = 1; index < flash_count; index++)
    {

    // set CS to SPI select channel 3 (FLASH)
    SPI_SET_CS(SPI_CS_FLASH);

    // re-issue seq program command, but NO address needed
    SPI_WriteRead( SEQ_PROGRAM );
    // now data
    SPI_WriteRead( *Bytes++ );

    // set CS to SPI select channel 0 (null)
    SPI_SET_CS(SPI_CS_0);

	// Delay to make sure the single byte was written
    DELAY_US(10);
    } // end for index

// set CS to SPI select channel 0 (null)
SPI_SET_CS(SPI_CS_0);

// finally send write disable command to signal end of sequential programming

// set CS to SPI select channel 3 (FLASH)
SPI_SET_CS(SPI_CS_FLASH);

SPI_WriteRead( WRITE_DISABLE );

// set CS to SPI select channel 0 (null)
SPI_SET_CS(SPI_CS_0);

// return success
return(1);

} // end Flash_Write

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Flash_Read(unsigned long Address, unsigned char *Bytes, unsigned int NumBytes)
{
// The Address argument is the flat memory address 0x00000->0xFFFFF in which to read. The
// Bytes array must be provided already by the calling function and will be filled upon
// returning if the flash read was successfuly. 


// Simple test for address out of bounds
if(Address > 0xFFFFF) return 0;

unsigned long flash_start_addr = Address;
unsigned int flash_count = NumBytes; 
unsigned int index;

// set CS to SPI select channel 3 (FLASH)
SPI_SET_CS(SPI_CS_FLASH);

// initiate read command
SPI_WriteRead( READ_DATA );

// write starting address byte by byte, MSB first...
SPI_WriteRead( flash_start_addr >> 16 );
SPI_WriteRead( flash_start_addr >> 8 );
SPI_WriteRead( flash_start_addr >> 0 );

// now read data 
for (index = 0; index < flash_count; index++)
    {
    *Bytes = SPI_WriteRead( 0x00 );
	Bytes++;
    } // end for index

// set CS to SPI select channel 0 (null)
SPI_SET_CS(SPI_CS_0);

// return success 
return(1);

} // end Flash_Read 

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Flash_Erase(unsigned int Block)
{
// The Flash_Erase function erases a single 4K byte block of memory. Remember when working
// with flash chips you need to first erase and then write. Overwritting a previously
// programmed section of memory won't work.


// step 1: send WRITE_ENABLE command

// set CS to SPI select channel 3 (FLASH)
SPI_SET_CS(SPI_CS_FLASH);
SPI_WriteRead( WRITE_ENABLE );

// disable SPI interface
// set CS to SPI select channel 0 (null)
SPI_SET_CS(SPI_CS_0);

DELAY_MS(10);


// step 2: block erase 4k bytes
unsigned long flash_start_addr = Block << 12;

// set CS to SPI select channel 3 (FLASH)
SPI_SET_CS(SPI_CS_FLASH);

SPI_WriteRead( BLOCK_ERASE4 );

// write block address, there are only a handful of blocks since each is 4-64K
SPI_WriteRead( flash_start_addr >> 16 );
SPI_WriteRead( flash_start_addr >> 8 );
SPI_WriteRead( flash_start_addr >> 0 );

// set CS to SPI select channel 0 (null)
SPI_SET_CS(SPI_CS_0);

// wait for the erase to finish
DELAY_MS(200);

// return success 
return(1);

} // end Flash_Erase

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
