///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// Copyright Nurve Networks LLC 2009
// 
// Filename: CHAM_PIC_FLASH_DRV_V010.h
//
// Original Author: Andre' LaMothe
//                  Joshua Hintze
// 
// Last Modified: 8.20.08
//
// Description: This file contains generic defines, macros, types for all the library modules and system related 
// code and utility functions.
//
// 
// 
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES ///////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// watch for multiple inclusions
#ifndef CHAM_PIC_FLASH_DRV
#define CHAM_PIC_FLASH_DRV
// support C++ compilers
#ifdef __cplusplus
extern "C" {
#endif 

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MACROS /////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// This macro is used in the Flash_Erase() function because
// we specifiy which block we are erasing and each block
// is 4KB in size.
#define FLASH_ADDR_TO_BLOCK(addr) (addr >> 12)

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES/////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// set the define for the proper FLASH installed on Chameleon
#define AT26F004         4 // numeric codes for flash version
#define AT26DF081A       8

// set type of FLASH ROM currently installed in Chameleon (8MBit) for conditional compilation of code
#define FLASH_ROM        AT26DF081A // AT26F004, AT26DF081A

// Atmel 2F004/8xxx series basic commands
#define WRITE_ENABLE     0x06
#define WRITE_DISABLE    0x04
#define READ_STATUS      0x05
#define WRITE_STATUS     0x01
#define READ_DATA        0x03

#define SEQ_PROGRAM      0xAF  
#define BYTE_PROGRAM     0x02 

#define BLOCK_ERASE4     0x20
#define BLOCK_ERASE32    0x52
#define BLOCK_ERASE64    0xD8

#define SECTOR_ERASE     0x20    

#define CHIP_ERASE       0x60 
#define JEDEC_ID         0x9F

#define PROTECT_SECTOR   0x36 
#define UNPROTECT_SECTOR 0x39 

// custom commands, not part of chip spec, just used here, start at 0xF0
#define UNPROTECT_CHIP   0xF0

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// TYPES/CLASSES //////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS ////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// conditional compilation for FLASH memory variants
#if (FLASH_ROM == AT26F004)

// we could use PGM memory for these and or do it algorithmically, but this is cleaner for now
// 11 sectors [64K, 64K, 64K, 64K, 64K, 64K, 64K, 32K, 8K, 8k, 16K ]
//            |<------- (7) 64K sectors ------->|  1      2     1 

    #define NUM_FLASH_ROM_SECTORS 11

    extern long sector_addresses[NUM_FLASH_ROM_SECTORS];

#endif


#if (FLASH_ROM == AT26DF081A)

// we could use PGM memory for these and or do it algorithmically, but this is cleaner for now
// 19 sectors [64K, 64K, 64K, ..., 64K, 64K, 64K, 64K, 16K, 8K, 8k, 32K ]
//            |<----------- (15) 64K sectors -------->| 1     2     1 

    #define NUM_FLASH_ROM_SECTORS 19

    extern long sector_addresses[NUM_FLASH_ROM_SECTORS];

#endif


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// EXTERNALS //////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// PROTOTYPES /////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Flash_Open();
int Flash_Close();
int Flash_Write(unsigned long Address, unsigned char *Bytes, unsigned int NumBytes);
int Flash_Read(unsigned long Address, unsigned char *Bytes, unsigned int NumBytes);
int Flash_Erase(unsigned int Block);

// end support for C++ compilers
#ifdef __cplusplus
}
#endif 


// end multiple inclusions 
#endif
