///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// Copyright Nurve Networks LLC 2009
// 
// Filename: CHAM_PIC_KEYBOARD_DRV_V010.C
//
// Original Author: Andre' LaMothe
//                  Joshua Hintze
// 
// Last Modified: 8.28.09
//
// Description: Keyboard API
//  
// Overview:
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES ///////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// include local Chameleon PIC API header files
#include "CHAM_PIC_SYSTEM_V010.h"
#include "CHAM_PIC_I2C_SPI_DRV_V010.h"
#include "CHAM_PIC_KEYBOARD_DRV_V010.h"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MACROS /////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES/////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// TYPES/CLASSES //////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS ////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// EXTERNALS //////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// PROTOTYPES /////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCTIONS  /////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// keyboard  support
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Keyboard_Key(void)
{
// retrieves the next key in the queue
int result;

// request keyboard read from slave
SPI_Prop_Send_Cmd( KEY_CMD_KEY, 0x00, 0x00); 

// wait a bit, driver takes time to respond...
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// keyboard should have enough time to be read now, request key by performing general read back command
// spi system will send back data in "result" buffer 

// return the keycode or 0 if no key ready
result = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00);
// wait a bit, driver takes time to respond...
//DELAY_US(SPI_PROP_DELAY_LONG_US);

return ( result );

} // Keyboard_Key

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Keyboard_State(int key)
{
// this tests the sent key's state to determine if its up/down
int result;

// request keyboard state test from slave 
SPI_Prop_Send_Cmd( KEY_CMD_KEYSTATE, key, 0x00); 

// wait a bit, driver takes time to respond...
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// keyboard should have enough time to be read now, request key by performing general read back command
// spi system will send back data in "result" buffer 

// return the keycode or 0 if no key ready (driver returns -1 for true, 0 for false, invert to normal TRUE/FALSE)
result = -SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00);
// wait a bit, driver takes time to respond...
//DELAY_US(SPI_PROP_DELAY_LONG_US);

return ( result );

} // end Keyboard_State

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Keyboard_GotKey(void)
{
// this test if a key is waiting in queue
int result;

// request keyboard state test from slave
SPI_Prop_Send_Cmd( KEY_CMD_GOTKEY, 0x00, 0x00); 

// wait a bit, driver takes time to respond...
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// keyboard should have enough time to be read now, request key by performing general read back command
// spi system will send back data in "result" buffer 
result = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00);
// wait a bit, driver takes time to respond...
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return the boolean if a key was ready
return ( result );

} // end Keyboard_GotKey

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Keyboard_Load(void)
{
// this loads the keyboard driver in the Prop
SPI_Prop_Send_Cmd(  KEY_CMD_START, 0x00, 0x00);

// wait a bit, driver takes time to respond...
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success for now
return(1);

} // end Keyboard_Load

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int Keyboard_Unload(void)
{
// this unloads the keyboard driver from the Prop
SPI_Prop_Send_Cmd(  KEY_CMD_STOP, 0x00, 0x00);

// wait a bit, driver takes time to respond...
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success for now
return(1);

} // end Keyboard_Unload

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
