///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// Copyright Nurve Networks LLC 2009
// 
// Filename: CHAM_PIC_PROP_PORT_DRV_V010.C
//
// Original Author: Andre' LaMothe
// 
// Last Modified: 8.28.09
//
// Description: Propeller Port API to communicate with the local 8-bit Port connected to the Propeller chip. 
//  
// Overview:
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES ///////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// include local Chameleon PIC API header files
#include "CHAM_PIC_SYSTEM_V010.h"
#include "CHAM_PIC_I2C_SPI_DRV_V010.h"
#include "CHAM_PIC_PROP_PORT_DRV_V010.h"


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MACROS /////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES/////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// TYPES/CLASSES //////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS ////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// EXTERNALS //////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// PROTOTYPES /////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCTIONS  /////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Propeller data port support
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int PropPort_SetDir(int dirbits)
{
// sets the Propeller's local 8-bit data port I/O directions

// 8-bit value to write as direction, 1=output, 0=input, eg: 11110000 = out, out, out, out, in, in, in, in
         
// send setdir port command to prop
SPI_Prop_Send_Cmd( PORT_CMD_SETDIR, dirbits, 0x00);

// wait a bit, driver takes time to respond...
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end PropPort_SetDir

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int PropPort_Read(void)
{
// reads the data from the 8-bit Propeller Port, I/Os set as outputs you should ignore
int result;

// send read port command to prop
SPI_Prop_Send_Cmd( PORT_CMD_READ, 0x00, 0x00);
//DELAY_US(SPI_PROP_DELAY_LONG_US);
                           
// now retrieve results, note, if we read the results back too soon, the driver might not have time to 
// set them, so keep that in mind
result = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00) & 0xFF;

// wait a bit, driver takes time to respond...
//DELAY_US(SPI_PROP_DELAY_LONG_US);

return ( result );

} // end PropPort_Read

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int PropPort_Write(int data8)
{
// writes to the 8-bit Propeller port, I/Os set as inputs will ignore bits

// send write port command to prop
SPI_Prop_Send_Cmd( PORT_CMD_WRITE, data8, 0x00);

// wait a bit, driver takes time to respond...
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end PropPort_Write


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
